define(["require", "exports", "crypto"], function (require, exports, crypto) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.decryptRSA = exports.encryptRSA = exports.decryptAES = exports.encryptAES = exports.decodeBase64 = exports.encodeBase64 = exports.sha256 = exports.md5 = void 0;
    function md5(data) {
        return crypto.createHash('md5').update(data).digest('hex');
    }
    exports.md5 = md5;
    function sha256(data, inputEncoding, outputEncoding) {
        if (inputEncoding === void 0) { inputEncoding = 'utf8'; }
        if (outputEncoding === void 0) { outputEncoding = 'hex'; }
        return crypto.createHash('sha256').update(data, inputEncoding).digest(outputEncoding);
    }
    exports.sha256 = sha256;
    function encodeBase64(data) {
        if (!data)
            return data;
        return Buffer.from(data).toString('base64');
    }
    exports.encodeBase64 = encodeBase64;
    function decodeBase64(data) {
        if (!data)
            return data;
        return Buffer.from(data, 'base64').toString();
    }
    exports.decodeBase64 = decodeBase64;
    function encryptAES(buf, key, iv, inputEncoding, outputEncoding, mode, keySize) {
        if (mode === void 0) { mode = 'cbc'; }
        var algorithm = 'aes-' + (keySize ? keySize : key.length * 8) + '-' + mode;
        var cipher = crypto.createCipheriv(algorithm, key, iv);
        var encrypted = cipher.update(buf, inputEncoding, outputEncoding);
        encrypted += cipher.final(outputEncoding);
        return encrypted;
    }
    exports.encryptAES = encryptAES;
    function decryptAES(buf, key, iv, inputEncoding, outputEncoding, mode, keySize) {
        if (mode === void 0) { mode = 'cbc'; }
        var algorithm = 'aes-' + (keySize ? keySize : key.length * 8) + '-' + mode;
        var decipher = crypto.createDecipheriv(algorithm, key, iv);
        var decrypted = decipher.update(buf, inputEncoding, outputEncoding);
        decrypted += decipher.final(outputEncoding);
        return decrypted;
    }
    exports.decryptAES = decryptAES;
    // type BufferEncoding = "ascii" | "utf8" | "utf-8" | "utf16le" | "ucs2" | "ucs-2" | "base64" | "latin1" | "binary" | "hex";
    function encryptRSA(buf, publicKey, inputEncoding, outputEncoding) {
        var _buf = Buffer.from(buf, inputEncoding);
        var encrypted = crypto.publicEncrypt(publicKey, _buf);
        return encrypted.toString(outputEncoding);
    }
    exports.encryptRSA = encryptRSA;
    function decryptRSA(buf, publicKey, inputEncoding, outputEncoding) {
        var _buf = Buffer.from(buf, inputEncoding);
        var decrypted = crypto.publicDecrypt(publicKey, _buf);
        return decrypted.toString(outputEncoding);
    }
    exports.decryptRSA = decryptRSA;
});
