define(["require", "exports", "chord/platform/log/common/log", "chord/platform/utils/common/paths", "process", "path", "electron", "chord/code/electron-main/menu/template", "chord/base/common/platform"], function (require, exports, log_1, paths_1, process, path, electron_1, template_1, platform_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    var logger = new log_1.Logger(paths_1.filenameToNodeName(__filename));
    logger.info('electron main active');
    var DEV = process.env.ELECTRON_DEV ? true : false;
    logger.info("[env]: ELECTRON_DEV : " + DEV);
    var win;
    // Quit when all windows are closed.
    electron_1.app.on('window-all-closed', function () {
        logger.info('[event]: window-all-closed');
        // On OS X it is common for applications and their menu bar
        // to stay active until the user quits explicitly with Cmd + Q
        if (!platform_1.isMacintosh) {
            electron_1.app.quit();
        }
    });
    electron_1.app.on('before-quit', function (event) {
        logger.info('[event]: before-quit');
    });
    electron_1.app.on('activate', function () {
        // On OS X it"s common to re-create a window in the app when the
        // dock icon is clicked and there are no other windows open.
        if (win === null) {
            logger.info('[event]: create window');
            createWindow();
        }
    });
    function createWindow() {
        win = new electron_1.BrowserWindow({
            height: 800,
            width: 1150,
            // For mac, linux, frameless
            titleBarStyle: 'hiddenInset',
            // For windows, frameless
            frame: platform_1.isWindows ? false : true,
            webPreferences: {
                nodeIntegration: true,
                enableRemoteModule: true,
                contextIsolation: false
            },
        });
        // path: chord/workbench/electron-browser/bootstrap/index.html
        win.loadFile(path.join(__dirname, '../../workbench/electron-browser/bootstrap/index.html'));
        if (DEV) {
            win.webContents.openDevTools();
        }
        win.on('close', function () {
            logger.info('[event]: window close');
            win.webContents.send('chord-quit', null);
            win = null;
        });
        // set menu
        electron_1.Menu.setApplicationMenu(electron_1.Menu.buildFromTemplate(template_1.menuTemplate(win)));
    }
    createWindow();
});
