define(["require", "exports", "path", "chord/preference/common/chord", "chord/base/common/assert", "fs", "chord/library/auth/encrypt", "chord/library/data/database", "chord/library/core/createTables"], function (require, exports, paths, chord_1, assert_1, fs, encrypt_1, database_1, createTables_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.defaultLibrary = exports.Library = void 0;
    var Library = /** @class */ (function () {
        function Library(username, databasePath, password) {
            if (password === void 0) { password = ''; }
            this.username = username;
            this.databasePath = databasePath;
            this.password = password;
            this.init();
        }
        Library.prototype.init = function () {
            if (fs.existsSync(this.databasePath)) {
                this.db = new database_1.LibraryDatabase(this.databasePath);
                // create and check database table schemes
                createTables_1.createTables(this.db);
                // auth
                var encPasswd1 = this.db.getEncryptedPassword();
                var encPasswd2 = encrypt_1.encryptPassword(this.password);
                assert_1.ok(encPasswd1 == encPasswd2, "[user] password not match: " + encPasswd1 + " != " + encPasswd2);
            }
            else {
                this.db = new database_1.LibraryDatabase(this.databasePath);
                createTables_1.createTables(this.db);
                this.db.addUser(this.username, encrypt_1.encryptPassword(this.password));
            }
        };
        Library.prototype.librarySongs = function (lastId, size, keyword) {
            if (lastId === void 0) { lastId = 0; }
            if (size === void 0) { size = 20; }
            return this.db.librarySongs(lastId, size, keyword);
        };
        Library.prototype.libraryAlbums = function (lastId, size, keyword) {
            if (lastId === void 0) { lastId = 0; }
            if (size === void 0) { size = 20; }
            return this.db.libraryAlbums(lastId, size, keyword);
        };
        Library.prototype.libraryArtists = function (lastId, size, keyword) {
            if (lastId === void 0) { lastId = 0; }
            if (size === void 0) { size = 20; }
            return this.db.libraryArtists(lastId, size, keyword);
        };
        Library.prototype.libraryCollections = function (lastId, size, keyword) {
            if (lastId === void 0) { lastId = 0; }
            if (size === void 0) { size = 20; }
            return this.db.libraryCollections(lastId, size, keyword);
        };
        Library.prototype.libraryUserProfiles = function (lastId, size, keyword) {
            if (lastId === void 0) { lastId = 0; }
            if (size === void 0) { size = 20; }
            return this.db.libraryUserProfiles(lastId, size, keyword);
        };
        // libraryPlayLists(lastId: number = 0, size: number = 20, keyword?: string): Array<IUserPlayList> {
        // let rows = this.db.libraryPlayLists(lastId, size, keyword);
        // let libraryPlayLists = rows.map(row => makeUserPlayList(row));
        // return libraryPlayLists;
        // }
        Library.prototype.libraryEpisodes = function (lastId, size, keyword) {
            if (lastId === void 0) { lastId = 0; }
            if (size === void 0) { size = 20; }
            return this.db.libraryEpisodes(lastId, size, keyword);
        };
        Library.prototype.libraryPodcasts = function (lastId, size, keyword) {
            if (lastId === void 0) { lastId = 0; }
            if (size === void 0) { size = 20; }
            return this.db.libraryPodcasts(lastId, size, keyword);
        };
        Library.prototype.libraryRadios = function (lastId, size, keyword) {
            if (lastId === void 0) { lastId = 0; }
            if (size === void 0) { size = 20; }
            return this.db.libraryRadios(lastId, size, keyword);
        };
        Library.prototype.addUser = function (username, password) {
            return this.db.addUser(username, password);
        };
        Library.prototype.addSong = function (song) {
            var addAt = Date.now();
            return this.db.addSong(song, addAt);
        };
        Library.prototype.addAlbum = function (album) {
            var addAt = Date.now();
            return this.db.addAlbum(album, addAt);
        };
        Library.prototype.addArtist = function (artist) {
            var addAt = Date.now();
            return this.db.addArtist(artist, addAt);
        };
        Library.prototype.addCollection = function (collection) {
            var addAt = Date.now();
            return this.db.addCollection(collection, addAt);
        };
        Library.prototype.addUserProfile = function (userProfile) {
            var addAt = Date.now();
            return this.db.addUserProfile(userProfile, addAt);
        };
        Library.prototype.addEpisode = function (episode) {
            var addAt = Date.now();
            return this.db.addEpisode(episode, addAt);
        };
        Library.prototype.addPodcast = function (podcast) {
            var addAt = Date.now();
            return this.db.addPodcast(podcast, addAt);
        };
        Library.prototype.addRadio = function (radio) {
            var addAt = Date.now();
            return this.db.addRadio(radio, addAt);
        };
        Library.prototype.deleteSong = function (song) {
            return this.db.deleteSong(song);
        };
        Library.prototype.deleteAlbum = function (album) {
            return this.db.deleteAlbum(album);
        };
        Library.prototype.deleteArtist = function (artist) {
            return this.db.deleteArtist(artist);
        };
        Library.prototype.deleteCollection = function (collection) {
            return this.db.deleteCollection(collection);
        };
        Library.prototype.deleteUserProfile = function (userProfile) {
            return this.db.deleteUserProfile(userProfile);
        };
        Library.prototype.deleteEpisode = function (episode) {
            return this.db.deleteEpisode(episode);
        };
        Library.prototype.deletePodcast = function (podcast) {
            return this.db.deletePodcast(podcast);
        };
        Library.prototype.deleteRadio = function (radio) {
            return this.db.deleteRadio(radio);
        };
        Library.prototype.exists = function (item) {
            return this.db.exists(item);
        };
        return Library;
    }());
    exports.Library = Library;
    // WARN: no using defaultLibrary for production
    var defaultDBPath = paths.join(chord_1.CHORD_DIR, 'default-library.db');
    exports.defaultLibrary = new Library('default', defaultDBPath);
});
