var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
define(["require", "exports", "cheerio", "chord/base/common/time", "chord/base/common/checker", "chord/music/common/origin", "chord/music/utils/lyric"], function (require, exports, cheerio, time_1, checker_1, origin_1, lyric_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.makeArtistAlbums = exports.extractArtistSongIds = exports.makeArtists = exports.makeArtist = exports.makeCollections = exports.extractCollectionSongIds = exports.makeCollection = exports.makeAlbums = exports.makeAlbum = exports.makeLyric = exports.makeSongs = exports.makeSong = exports.makeAudios = void 0;
    function absoluteUrl(uri) {
        if (uri.startsWith('//')) {
            return 'http:' + uri;
        }
        return uri;
    }
    var _origin = 'migu';
    var _getSongUrl = origin_1.getSongUrl.bind(null, _origin);
    var _getSongId = origin_1.getSongId.bind(null, _origin);
    var _getAlbumUrl = origin_1.getAlbumUrl.bind(null, _origin);
    var _getAlbumId = origin_1.getAlbumId.bind(null, _origin);
    var _getArtistUrl = origin_1.getArtistUrl.bind(null, _origin);
    var _getArtistId = origin_1.getArtistId.bind(null, _origin);
    var _getCollectionUrl = origin_1.getCollectionUrl.bind(null, _origin);
    var _getCollectionId = origin_1.getCollectionId.bind(null, _origin);
    // const _getUserUrl: (id: string) => string = getUserUrl.bind(null, _origin);
    var _getUserId = origin_1.getUserId.bind(null, _origin);
    function makeAudios(infos) {
        if (!infos)
            return null;
        var audios = [];
        var url = absoluteUrl(infos['playUrl']);
        var format;
        var kbps;
        if (url.includes('/flac/')) {
            kbps = 720;
            format = 'flac';
        }
        else {
            format = 'mp3';
            if (url.match(/MP3_\d+/)) {
                var m = /MP3_(\d+)/.exec(url)[1];
                kbps = Number.parseInt(m);
            }
            else {
                kbps = 90;
            }
        }
        var audio = {
            format: format,
            size: null,
            kbps: kbps,
            url: url,
            path: null,
        };
        audios.push(audio);
        return audios;
    }
    exports.makeAudios = makeAudios;
    function makeSong(info) {
        var e_1, _a;
        var songOriginalId = info['copyrightId'];
        var songMid = (info['songId'] || info['id']).toString();
        var albumInfo = info['albums'] ? info['albums'][0] : info;
        var albumOriginalId = albumInfo['albumId'] && albumInfo['albumId'].toString();
        var artistInfo = info['singers'] ? info['singers'][0] : info;
        var artistOriginalId = artistInfo['artistId']
            || artistInfo['singerId']
                && (checker_1.isObject(artistInfo['singerId']) ? (artistInfo['singerId'].length > 0 ? artistInfo['singerId'][0].toString() : null)
                    : artistInfo['singerId']);
        var artistName = artistInfo['artistName']
            || artistInfo['singerName']
                && (checker_1.isObject(artistInfo['singerName']) ? (artistInfo['singerName'].length > 0 ? artistInfo['singerName'][0].split(',')[0] : null)
                    : artistInfo['singerName']);
        var audios = [];
        if (checker_1.isObject(info['auditions'])) {
            try {
                for (var _b = __values(['HQ', 'SQ', 'Bq']), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var tp = _c.value;
                    var url = info['auditions']['lis' + tp];
                    var size = info['auditions']['lis' + tp + 'Size'];
                    if (!url)
                        continue;
                    var kbps = void 0;
                    if (url.includes('MP3_128')) {
                        kbps = 128;
                    }
                    else if (url.includes('MP3_320')) {
                        kbps = 320;
                    }
                    else if (url.includes('flac')) {
                        kbps = 720;
                    }
                    else {
                        continue;
                    }
                    var audio = {
                        format: url.split('.').slice(-1)[0],
                        size: size,
                        kbps: kbps,
                        url: url,
                        path: null,
                    };
                    audios.push(audio);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        var song = {
            songId: _getSongId(songOriginalId),
            type: 'song',
            origin: _origin,
            songOriginalId: songOriginalId,
            url: _getSongUrl(songOriginalId),
            songName: info['songName'] || (info['contentName'] || '').split(',')[0],
            songMid: songMid,
            albumId: _getAlbumId(albumOriginalId),
            albumOriginalId: albumOriginalId,
            albumName: albumInfo['albumName'] || info['cover'],
            albumCoverUrl: albumInfo['picS'],
            // so we use the smallest one
            artistId: _getArtistId(artistOriginalId),
            artistOriginalId: artistOriginalId,
            artistName: artistName,
            lyricUrl: info['lyrics'],
            track: null,
            cdSerial: null,
            // millisecond
            duration: info['length'] && time_1.parseToMillisecond(info['length']),
            // millisecond
            releaseDate: null,
            playCountWeb: 0,
            playCount: 0,
            audios: audios,
            disable: false,
        };
        return song;
    }
    exports.makeSong = makeSong;
    function makeSongs(info) {
        return (info || []).map(function (songInfo) { return makeSong(songInfo); });
    }
    exports.makeSongs = makeSongs;
    function makeLyric(songId, lyricInfo, transInfo) {
        if (!lyricInfo)
            return null;
        var lyric = lyric_1.makeLyric(lyricInfo);
        var chunksMap = {};
        lyric.chunks.forEach(function (chunk) { return chunksMap[chunk.point] = chunk; });
        if (transInfo) {
            var trans = lyric_1.makeLyric(transInfo);
            trans.chunks.forEach(function (chunk, index) {
                var lyricChunk = chunksMap[chunk.point];
                if (lyricChunk)
                    lyricChunk.translation = chunk.text;
            });
        }
        lyric.songId = _getSongId(songId);
        return lyric;
    }
    exports.makeLyric = makeLyric;
    function makeAlbum(info) {
        var albumOriginalId = (info['albumId'] || info['id']).toString();
        var albumCoverUrl = (info['localAlbumPicS'] || info['albumPicS'] || '').split('?')[0];
        var artistOriginalId = null;
        var artistName = null;
        if (info['singerId']) {
            artistOriginalId = info['singerId'].toString();
        }
        else {
            if (checker_1.isObject(info['singer'])) {
                artistOriginalId = info['singer'][0]['id'];
                artistName = info['singer'][0]['name'];
            }
        }
        var album = {
            albumId: _getAlbumId(albumOriginalId),
            type: 'album',
            origin: _origin,
            albumOriginalId: albumOriginalId,
            url: _getAlbumUrl(albumOriginalId),
            albumName: info['albumName'] || info['title'],
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(artistOriginalId),
            artistOriginalId: artistOriginalId,
            artistName: artistName,
            tags: [],
            description: info['albumIntro'],
            releaseDate: time_1.parseToMillisecond(info['publishDate']),
            company: info['publishCompany'],
            songs: [],
            songCount: Number.parseInt(info['trackCount'] || info['songNum']),
        };
        return album;
    }
    exports.makeAlbum = makeAlbum;
    function makeAlbums(info) {
        return (info || []).map(function (albumInfo) { return makeAlbum(albumInfo); });
    }
    exports.makeAlbums = makeAlbums;
    function makeCollection(info) {
        var collectionOriginalId = (info['playListId'] || info['id']).toString();
        var collectionCoverUrl = info['image'] || info['img'];
        var tags = (info['tagLists'] || []).map(function (tag) { return ({ name: tag['tagName'] }); });
        var userOriginalId = info['createUserId'] || info['userId'];
        var userName = info['createName'] || info['userName'];
        var collection = {
            collectionId: _getCollectionId(collectionOriginalId),
            type: 'collection',
            origin: _origin,
            collectionOriginalId: collectionOriginalId,
            url: _getCollectionUrl(collectionOriginalId),
            collectionName: info['playListName'] || info['name'],
            collectionCoverUrl: collectionCoverUrl,
            userId: _getUserId(userOriginalId),
            userName: userName,
            releaseDate: Date.parse(info['createTime']),
            description: info['summary'],
            tags: tags,
            songs: [],
            songCount: Number.parseInt(info['contentCount'] || info['musicNum']),
            playCount: info['playCount'] || info['playNum'],
            likeCount: info['keepNum'],
        };
        return collection;
    }
    exports.makeCollection = makeCollection;
    function extractCollectionSongIds(html) {
        var m = /data-cids="([\da-zA-Z]+,[\da-zA-Z,]+)"/.exec(html);
        var songIds = m ? m[1].split(',').filter(function (id) { return !!id; }) : [];
        return songIds;
    }
    exports.extractCollectionSongIds = extractCollectionSongIds;
    function makeCollections(info) {
        return (info || []).map(function (collectionInfo) { return makeCollection(collectionInfo); });
    }
    exports.makeCollections = makeCollections;
    function makeArtist(info) {
        var artistOriginalId = (info['artistId'] || info['id']).toString();
        var artistAvatarUrl = (info['localArtistPicS'] || info['artistPicS'] || '').split('?')[0];
        var artist = {
            artistId: _getArtistId(artistOriginalId),
            type: 'artist',
            origin: _origin,
            artistOriginalId: artistOriginalId,
            url: _getArtistUrl(artistOriginalId),
            artistName: info['artistName'] || info['title'],
            artistAlias: info['anotherName'],
            artistAvatarUrl: artistAvatarUrl,
            description: info['intro'],
            songs: [],
            albums: [],
            playCount: info['playCount'],
        };
        return artist;
    }
    exports.makeArtist = makeArtist;
    function makeArtists(info) {
        return (info || []).map(function (artistInfo) { return makeArtist(artistInfo); });
    }
    exports.makeArtists = makeArtists;
    function extractArtistSongIds(html) {
        var $ = cheerio.load(html);
        var songIds = Array.from(($('a.action') || [])).map(function (item) { return item.attribs['data-cids']; }).filter(function (id) { return !!id; });
        return songIds;
    }
    exports.extractArtistSongIds = extractArtistSongIds;
    function makeArtistAlbums(html) {
        var $ = cheerio.load(html);
        // Artist infos
        var elem = $('a.singer')[0];
        if (!elem)
            return [];
        var artistOriginalId = elem.attribs.href.split('/').slice(-1)[0];
        var artistName = elem.children[0].data;
        var description = $('div.full-content')[0].children[0].data;
        var albums = Array.from($('a.thumb-link')).map(function (item) {
            var albumOriginalId = item.attribs.href.split('/').slice(-1)[0];
            var img = item.children.filter(function (c) { return c.name == 'img'; })[0];
            var albumCoverUrl = absoluteUrl(img.attribs['data-original']);
            var albumName = img.attribs.alt;
            var album = {
                albumId: _getAlbumId(albumOriginalId),
                type: 'album',
                origin: _origin,
                albumOriginalId: albumOriginalId,
                url: _getAlbumUrl(albumOriginalId),
                albumName: albumName,
                albumCoverUrl: albumCoverUrl,
                artistId: _getArtistId(artistOriginalId),
                artistOriginalId: artistOriginalId,
                artistName: artistName,
                tags: [],
                description: description,
                songs: [],
            };
            return album;
        });
        return albums;
    }
    exports.makeArtistAlbums = makeArtistAlbums;
});
