define(["require", "exports", "cheerio", "chord/music/common/origin", "chord/music/utils/lyric"], function (require, exports, cheerio, origin_1, lyric_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.getInfosFromHtml = exports.makeUserProfiles = exports.makeUserProfile = exports.makeArtistAlbums = exports.makeArtists = exports.makeArtist = exports.makeCollections = exports.makeCollection = exports.makeAlbums = exports.makeAlbum = exports.makeLyric = exports.makeSongs = exports.makeSong = exports.makeAudio = void 0;
    var _origin = 'netease';
    var _getSongUrl = origin_1.getSongUrl.bind(null, _origin);
    var _getSongId = origin_1.getSongId.bind(null, _origin);
    var _getAlbumUrl = origin_1.getAlbumUrl.bind(null, _origin);
    var _getAlbumId = origin_1.getAlbumId.bind(null, _origin);
    var _getArtistUrl = origin_1.getArtistUrl.bind(null, _origin);
    var _getArtistId = origin_1.getArtistId.bind(null, _origin);
    var _getCollectionUrl = origin_1.getCollectionUrl.bind(null, _origin);
    var _getCollectionId = origin_1.getCollectionId.bind(null, _origin);
    var _getUserUrl = origin_1.getUserUrl.bind(null, _origin);
    var _getUserId = origin_1.getUserId.bind(null, _origin);
    function makeAudio(info) {
        if (!info['url'])
            return null;
        return {
            format: info['type'],
            size: info['size'],
            kbps: Math.floor(info['br'] / 1000),
            url: info['url'],
            path: null,
        };
    }
    exports.makeAudio = makeAudio;
    function makeSong(info, privilege) {
        if (!privilege) {
            privilege = info['privilege'];
        }
        var songOriginalId = info['id'].toString();
        // info['album'] for similarSongs
        var albumInfo = info['al'] || info['album'];
        var artistInfo = (info['ar'] && info['ar'][0]) || (info['artists'] && info['artists'][0]);
        var albumCoverUrl = albumInfo['picUrl'];
        var disable = privilege['st'] != 0;
        var song = {
            songId: _getSongId(songOriginalId),
            type: 'song',
            origin: _origin,
            songOriginalId: songOriginalId,
            url: _getSongUrl(songOriginalId),
            songName: info['name'],
            albumId: _getAlbumId(albumInfo['id']),
            albumOriginalId: albumInfo['id'].toString(),
            albumName: albumInfo['name'],
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(artistInfo['id']),
            artistOriginalId: artistInfo['id'].toString(),
            artistName: artistInfo['name'],
            track: info['no'],
            cdSerial: info['cd'] ? parseInt(info['cd']) : (info['disc'] ? parseInt(info['disc']) : 1),
            // millisecond
            duration: info['dt'] || info['duration'],
            // millisecond
            releaseDate: info['publishTime'] || albumInfo['publishTime'],
            playCountWeb: info['cp'],
            playCount: 0,
            audios: [],
            disable: disable,
        };
        return song;
    }
    exports.makeSong = makeSong;
    function makeSongs(info) {
        return (info || []).map(function (songInfo) { return makeSong(songInfo); });
    }
    exports.makeSongs = makeSongs;
    function makeLyric(songId, lyricInfo, transInfo) {
        if (!lyricInfo)
            return null;
        var lyric = lyric_1.makeLyric(lyricInfo);
        var chunksMap = {};
        lyric.chunks.forEach(function (chunk) { return chunksMap[chunk.point] = chunk; });
        if (transInfo) {
            var trans = lyric_1.makeLyric(transInfo);
            trans.chunks.forEach(function (chunk, index) {
                var lyricChunk = chunksMap[chunk.point];
                if (lyricChunk)
                    lyricChunk.translation = chunk.text;
            });
        }
        lyric.songId = _getSongId(songId);
        return lyric;
    }
    exports.makeLyric = makeLyric;
    function makeAlbum(info) {
        // artist's albums have not `album`
        var albumInfo = info['album'] || info;
        var songsInfo = info['songs'];
        var albumOriginalId = albumInfo['id'].toString();
        var albumCoverUrl = albumInfo['picUrl'];
        var songs = (songsInfo || []).map(function (info) {
            var song = makeSong(info);
            // song info does not provide its album cover url
            song.albumCoverUrl = albumCoverUrl;
            return song;
        });
        var album = {
            albumId: _getAlbumId(albumOriginalId),
            type: 'album',
            origin: _origin,
            albumOriginalId: albumOriginalId,
            url: _getAlbumUrl(albumOriginalId),
            albumName: albumInfo['name'],
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(albumInfo['artist']['id']),
            artistOriginalId: albumInfo['artist']['id'].toString(),
            artistName: albumInfo['artist']['name'],
            tags: [],
            description: albumInfo['description'] || null,
            releaseDate: albumInfo['publishTime'],
            company: albumInfo['company'],
            songs: songs,
            songCount: albumInfo['size'],
        };
        return album;
    }
    exports.makeAlbum = makeAlbum;
    function makeAlbums(info) {
        return (info || []).map(function (albumInfo) { return makeAlbum(albumInfo); });
    }
    exports.makeAlbums = makeAlbums;
    function makeCollection(info, privileges) {
        if (privileges === void 0) { privileges = []; }
        var collectionOriginalId = info['id'].toString();
        var collectionCoverUrl = info['coverImgUrl'] || info['picUrl'];
        var tags = (info['tags'] || []).map(function (tag) { return ({ name: tag }); });
        var songs = (info['tracks'] || []).map(function (songInfo, index) { return makeSong(songInfo, privileges[index]); });
        var duration = songs.length != 0 ? songs.map(function (s) { return s.duration; }).reduce(function (x, y) { return x + y; }) : null;
        var userOriginalId = info['creator'] ? info['creator']['userId'].toString() : info['userId'] ? info['userId'].toString() : null;
        var userName = info['creator'] ? info['creator']['nickname'] : null;
        var collection = {
            collectionId: _getCollectionId(collectionOriginalId),
            type: 'collection',
            origin: _origin,
            collectionOriginalId: collectionOriginalId,
            url: _getCollectionUrl(collectionOriginalId),
            collectionName: info['name'],
            collectionCoverUrl: collectionCoverUrl,
            userId: _getUserId(userOriginalId),
            userName: userName,
            releaseDate: info['createTime'],
            description: info['description'] || info['copywriter'],
            tags: tags,
            duration: duration,
            songs: songs,
            songCount: info['trackCount'],
            playCount: info['playCount'] || info['playcount'],
            likeCount: info['subscribedCount'],
        };
        return collection;
    }
    exports.makeCollection = makeCollection;
    function makeCollections(info) {
        return (info || []).map(function (collectionInfo) { return makeCollection(collectionInfo); });
    }
    exports.makeCollections = makeCollections;
    function makeArtist(info) {
        var artistInfo = info['artist'] || info;
        var songsInfo = info['hotSongs'] || [];
        var artistOriginalId = artistInfo['id'].toString();
        var artistAvatarUrl = artistInfo['picUrl'];
        // let artistAlias = info['alias'].split('/').filter(a => a.trim() != '').map(a => a.trim());
        var songs = songsInfo.map(function (songInfo) { return makeSong(songInfo); });
        var artist = {
            artistId: _getArtistId(artistOriginalId),
            type: 'artist',
            origin: _origin,
            artistOriginalId: artistOriginalId,
            url: _getArtistUrl(artistOriginalId),
            artistName: artistInfo['name'],
            // artistAlias: artistAlias,
            artistAvatarUrl: artistAvatarUrl,
            description: artistInfo['briefDesc'],
            songs: songs,
            albums: [],
            playCount: info['playCount'],
        };
        return artist;
    }
    exports.makeArtist = makeArtist;
    function makeArtists(info) {
        return (info || []).map(function (artistInfo) { return makeArtist(artistInfo); });
    }
    exports.makeArtists = makeArtists;
    function makeArtistAlbums(info) {
        return (info['hotAlbums'] || []).map(function (albumInfo) { return makeAlbum(albumInfo); });
    }
    exports.makeArtistAlbums = makeArtistAlbums;
    function makeUserProfile(info, userId) {
        var userOriginalId = userId || info['userId'].toString();
        var user = {
            userId: _getUserId(userOriginalId),
            type: 'userProfile',
            origin: _origin,
            userOriginalId: userOriginalId,
            url: _getUserUrl(userOriginalId),
            userName: info['nickname'],
            userAvatarUrl: info['avatarImg'] || info['avatarUrl'],
            listenCount: info['playCount'],
            followingCount: info['follows'],
            followerCount: info['followeds'],
            favoriteCollectionCount: info['playlistCount'],
            description: info['signature'],
        };
        return user;
    }
    exports.makeUserProfile = makeUserProfile;
    function makeUserProfiles(info) {
        return (info || []).map(function (userProfile) { return makeUserProfile(userProfile); });
    }
    exports.makeUserProfiles = makeUserProfiles;
    function getInfosFromHtml(html) {
        var $ = cheerio.load(html);
        var userOriginalId = /window.hostId\s*=\s*(\d+)/.exec(html)[1];
        var info = {
            userId: _getUserId(userOriginalId),
            followingCount: parseInt($('#follow_count').text().trim()),
            followerCount: parseInt($('#fan_count').text().trim()),
            favoriteCollectionCount: parseInt(/cCount\s*:\s*(\d+)/g.exec(html)[1]),
        };
        return info;
    }
    exports.getInfosFromHtml = getInfosFromHtml;
});
