define(["require", "exports", "chord/base/node/url", "chord/music/common/origin", "chord/music/utils/lyric"], function (require, exports, url_1, origin_1, lyric_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.makeAccount = exports.makeUserProfileMore = exports.makeUserProfiles = exports.makeUserProfile = exports.makeAliCollections = exports.makeAliCollection = exports.makeAliArtists = exports.makeAliArtist = exports.makeAliAlbums = exports.makeAliAlbum = exports.makeAliSongs = exports.makeAliSong = exports.makeArtists = exports.makeArtist = exports.makeCollections = exports.makeCollection = exports.makeAlbums = exports.makeAlbum = exports.makeAliLyric = exports.makeSongs = exports.makeSong = void 0;
    var _staticResourceBasicUrl = 'http://img.xiami.net';
    var _origin = 'xiami';
    var _getSongUrl = origin_1.getSongUrl.bind(null, _origin);
    var _getSongId = origin_1.getSongId.bind(null, _origin);
    var _getAlbumUrl = origin_1.getAlbumUrl.bind(null, _origin);
    var _getAlbumId = origin_1.getAlbumId.bind(null, _origin);
    var _getArtistUrl = origin_1.getArtistUrl.bind(null, _origin);
    var _getArtistId = origin_1.getArtistId.bind(null, _origin);
    var _getCollectionUrl = origin_1.getCollectionUrl.bind(null, _origin);
    var _getCollectionId = origin_1.getCollectionId.bind(null, _origin);
    var _getUserUrl = origin_1.getUserUrl.bind(null, _origin);
    var _getUserId = origin_1.getUserId.bind(null, _origin);
    /**
     * All images from 'http://img.xiami.net' and 'http://pic.xiami.net/'
     * can be resize by adding follow params
     * ?x-oss-process=image/resize,m_fill,w_382,h_382/format,webp
     */
    function makeSong(info) {
        var songInfo = info['songDetail'] || info;
        var songExtInfo = info['songExt'] || info;
        var lyricUrl = songInfo['lyricInfo'] ? songInfo['lyricInfo']['lyricFile'] : songInfo['lyric'];
        var audios = [];
        if (songInfo['listenFiles'] && songInfo['listenFiles'].length > 0) {
            audios = songInfo['listenFiles'].map(function (a) { return makeAudio(a); }).sort(function (x, y) { return y.kbps - x.kbps; });
        }
        else {
            // block song audio may be at here
            var backupSong = songInfo['bakSong'];
            if (backupSong && backupSong['listenFiles'] && backupSong['listenFiles'].length > 0) {
                audios = backupSong['listenFiles'].map(function (a) { return makeAudio(a); }).sort(function (x, y) { return y.kbps - x.kbps; });
            }
        }
        var styles = songExtInfo['songStyle'] ? (songExtInfo['songStyle']['styles'] || []).map(function (i) { return ({ id: i['id'].toString(), name: i['title'] }); }) : [];
        var tags = songExtInfo['songTag'] ? (songExtInfo['songTag']['tags'] || []).map(function (i) { return ({ id: i['id'].toString(), name: i['name'] }); }) : [];
        var songOriginalId = songInfo['songId'];
        var song = {
            songId: _getSongId(songOriginalId),
            type: 'song',
            origin: _origin,
            songOriginalId: songOriginalId,
            url: _getSongUrl(songOriginalId),
            songName: songInfo['songName'],
            subTitle: songInfo['subName'],
            songWriters: songInfo['songwriters'],
            singers: songInfo['singers'],
            albumId: _getAlbumId(songInfo['albumId']),
            albumOriginalId: (songInfo['albumId'] || '').toString(),
            albumName: songInfo['albumName'],
            albumCoverUrl: songInfo['albumLogo'],
            artistId: _getArtistId(songInfo['artistId']),
            artistOriginalId: (songInfo['artistId'] || '').toString(),
            artistName: songInfo['artistName'],
            artistAvatarUrl: songInfo['artistLogo'],
            composer: info['composer'],
            styles: styles,
            tags: tags,
            lyricUrl: lyricUrl,
            track: songInfo['track'],
            cdSerial: songInfo['cdSerial'],
            // millisecond
            duration: songInfo['length'],
            // millisecond
            releaseDate: songInfo['gmtPublish'] || songInfo['gmtCreate'] || (songExtInfo['album'] ? songExtInfo['album']['gmtPublish'] : null),
            playCountWeb: info['playCount'] || null,
            playCount: 0,
            audios: audios,
        };
        return song;
    }
    exports.makeSong = makeSong;
    function makeSongs(info) {
        return (info || []).map(function (songInfo) { return makeSong(songInfo); });
    }
    exports.makeSongs = makeSongs;
    function makeAliLyric(songId, info) {
        var lyric = lyric_1.makeLyric(info);
        lyric.songId = _getSongId(songId);
        return lyric;
    }
    exports.makeAliLyric = makeAliLyric;
    function makeAlbum(info) {
        var albumOriginalId = info['albumId'].toString();
        var albumId = _getAlbumId(albumOriginalId);
        var albumCoverUrl = url_1.getAbsolutUrl(info['albumLogo'], _staticResourceBasicUrl);
        var albumName = info['albumName'];
        var artistOriginalId = info['artistId'].toString();
        var artistId = _getArtistId(artistOriginalId);
        var artistName = info['artistName'];
        var releaseDate = info['gmtPublish'] || info['gmtCreate'];
        var duration = 0;
        var songs = (info['songs'] || []).map(function (i) { return makeSong(i); });
        songs.forEach(function (song) {
            duration += song.duration;
            if (!song.albumOriginalId) {
                song.albumOriginalId = albumOriginalId;
                song.albumId = albumId;
                song.albumCoverUrl = albumCoverUrl;
            }
            if (!song.artistOriginalId) {
                song.artistOriginalId = artistOriginalId;
                song.artistId = artistId;
            }
        });
        var album = {
            albumId: _getAlbumId(albumOriginalId),
            type: 'album',
            origin: _origin,
            albumOriginalId: albumOriginalId,
            url: _getAlbumUrl(albumOriginalId),
            albumName: albumName,
            albumCoverUrl: albumCoverUrl,
            artistId: artistId,
            artistOriginalId: artistOriginalId,
            artistName: artistName,
            duration: duration,
            releaseDate: releaseDate,
            songs: songs,
            songCount: songs.length,
        };
        return album;
    }
    exports.makeAlbum = makeAlbum;
    function makeAlbums(info) {
        return (info || []).map(function (albumInfo) { return makeAlbum(albumInfo); });
    }
    exports.makeAlbums = makeAlbums;
    function makeCollection(info) {
        var collectionOriginalId = info['listId'];
        var collectionId = _getCollectionId(collectionOriginalId);
        var collectionName = info['collectName'];
        var collectionCoverUrl = url_1.getAbsolutUrl(info['collectLogo'], _staticResourceBasicUrl);
        var tags = (info['tags'] || []).map(function (tag) { return ({ name: tag }); });
        var songs = (info['songs'] || []).map(function (songInfo) { return makeAliSong(songInfo); });
        var duration = songs.length != 0 ? songs.map(function (s) { return s.duration; }).reduce(function (x, y) { return x + y; }) : null;
        var collection = {
            collectionId: collectionId,
            type: 'collection',
            origin: _origin,
            collectionOriginalId: collectionOriginalId,
            url: _getCollectionUrl(collectionOriginalId),
            collectionName: collectionName,
            collectionCoverUrl: collectionCoverUrl,
            userId: _getUserId((info['userId'] || '').toString()),
            userName: info['userName'],
            // TODO: updateDate
            // updateDate: info['gmtModify'],
            releaseDate: info['gmtCreate'],
            description: info['description'],
            tags: tags,
            duration: duration,
            songs: songs,
            songCount: info['songCount'],
            playCount: info['playCount'],
            likeCount: info['collects'],
        };
        return collection;
    }
    exports.makeCollection = makeCollection;
    function makeCollections(info) {
        return (info || []).map(function (collectionInfo) { return makeCollection(collectionInfo); });
    }
    exports.makeCollections = makeCollections;
    function makeArtist(info) {
        var artistOriginalId = info['artistId'];
        var artistId = _getArtistId(artistOriginalId);
        var artistAvatarUrl = url_1.getAbsolutUrl(info['artistLogo'], _staticResourceBasicUrl);
        var artistAlias = info['alias'].split('/').filter(function (a) { return a.trim() != ''; }).map(function (a) { return a.trim(); });
        var artist = {
            artistId: artistId,
            type: 'artist',
            origin: _origin,
            artistOriginalId: artistOriginalId,
            url: _getArtistUrl(artistOriginalId),
            artistName: info['artistName'],
            artistAlias: artistAlias,
            artistAvatarUrl: artistAvatarUrl,
            area: info['area'],
            description: info['description'],
            songs: [],
            albums: [],
            playCount: info['playCount'],
            likeCount: info['countLikes'],
        };
        return artist;
    }
    exports.makeArtist = makeArtist;
    function makeArtists(info) {
        return (info || []).map(function (artistInfo) { return makeArtist(artistInfo); });
    }
    exports.makeArtists = makeArtists;
    function getKbps(str) {
        var r = /\/\/\w(\d{2,3})\./.exec(str.slice(0, 20));
        if (r) {
            return parseInt(r[1]);
        }
        else {
            return 128;
        }
    }
    function makeAudio(info) {
        var url = info['listenFile'] || info['url'];
        var audio = {
            format: info['format'],
            size: info['fileSize'] || info['filesize'],
            kbps: getKbps(url),
            url: url,
        };
        return audio;
    }
    function makeAliSong(info) {
        var lyricUrl;
        if (!!info['lyricInfo']) {
            lyricUrl = url_1.getAbsolutUrl(info['lyricInfo']['lyricFile'], _staticResourceBasicUrl);
        }
        var albumCoverUrl = url_1.getAbsolutUrl(info['albumLogo'], _staticResourceBasicUrl);
        var artistAvatarUrl = url_1.getAbsolutUrl(info['artistLogo'], _staticResourceBasicUrl);
        var songOriginalId = info['songId'].toString();
        var audios = [];
        if (info['listenFiles'] && info['listenFiles'].length > 0) {
            audios = info['listenFiles'].map(function (a) { return makeAudio(a); }).sort(function (x, y) { return y.kbps - x.kbps; });
        }
        else {
            // block song audio may be at here
            var backupSong = info['bakSong'];
            if (backupSong && backupSong['listenFiles'] && backupSong['listenFiles'].length > 0) {
                audios = backupSong['listenFiles'].map(function (a) { return makeAudio(a); }).sort(function (x, y) { return y.kbps - x.kbps; });
            }
        }
        var song = {
            songId: _getSongId(songOriginalId),
            type: 'song',
            origin: _origin,
            songOriginalId: songOriginalId,
            url: _getSongUrl(songOriginalId),
            songName: info['songName'],
            subTitle: info['subName'],
            songWriters: info['songwriters'],
            singers: info['singers'],
            albumId: _getAlbumId(info['albumId']),
            albumOriginalId: info['albumId'].toString(),
            albumName: info['albumName'],
            albumCoverUrl: albumCoverUrl,
            artistId: _getArtistId(info['artistId']),
            artistOriginalId: (info['artistId'] || '').toString(),
            artistName: info['artistName'],
            artistAvatarUrl: artistAvatarUrl,
            composer: info['composer'],
            lyricUrl: lyricUrl,
            track: info['track'],
            cdSerial: info['cdSerial'],
            duration: info['length'],
            releaseDate: info['gmtCreate'],
            playCountWeb: info['playCount'],
            playCount: info['playCount'],
            description: info['description'],
            audios: audios,
        };
        return song;
    }
    exports.makeAliSong = makeAliSong;
    function makeAliSongs(info) {
        var songs = (info || []).map(function (songInfo) { return makeAliSong(songInfo); });
        return songs;
    }
    exports.makeAliSongs = makeAliSongs;
    function makeAliAlbum(info) {
        var albumOriginalId = info['albumId'].toString();
        var albumCoverUrl = url_1.getAbsolutUrl(info['albumLogo'], _staticResourceBasicUrl);
        var tags = [{ id: info['categoryId'], name: info['albumCategory'] }];
        var songs = (info['songs'] || []).map(function (song) { return makeAliSong(song); });
        var album = {
            albumId: _getAlbumId(albumOriginalId),
            type: 'album',
            origin: _origin,
            albumOriginalId: albumOriginalId,
            url: _getAlbumUrl(albumOriginalId),
            albumName: info['albumName'],
            albumCoverUrl: albumCoverUrl,
            subTitle: info['subName'],
            artistId: _getArtistId(info['artistId']),
            artistOriginalId: info['artistId'].toString(),
            artistName: info['artistName'],
            tags: tags,
            description: info['description'],
            releaseDate: info['gmtPublish'],
            company: info['company'],
            songs: songs,
            songCount: info['songCount'],
            playCount: info['playCount'],
            likeCount: info['collects'],
        };
        return album;
    }
    exports.makeAliAlbum = makeAliAlbum;
    function makeAliAlbums(info) {
        var albums = (info || []).map(function (albumInfo) { return makeAliAlbum(albumInfo); });
        return albums;
    }
    exports.makeAliAlbums = makeAliAlbums;
    function makeAliArtist(info) {
        var artistOriginalId = info['artistId'].toString();
        var artistAvatarUrl = url_1.getAbsolutUrl(info['artistLogo'], _staticResourceBasicUrl);
        var artistAlias = info['alias'] && info['alias'].split('/').filter(function (a) { return a.trim() != ''; }).map(function (a) { return a.trim(); });
        var artist = {
            artistId: _getArtistId(artistOriginalId),
            type: 'artist',
            origin: _origin,
            artistOriginalId: artistOriginalId,
            url: _getArtistUrl(artistOriginalId),
            artistName: info['artistName'],
            artistAlias: artistAlias,
            artistAvatarUrl: artistAvatarUrl,
            area: info['area'],
            description: info['description'],
            songs: [],
            albums: [],
            playCount: info['playCount'],
            likeCount: info['countLikes'],
        };
        return artist;
    }
    exports.makeAliArtist = makeAliArtist;
    function makeAliArtists(info) {
        var artists = (info || []).map(function (artistInfo) { return makeAliArtist(artistInfo); });
        return artists;
    }
    exports.makeAliArtists = makeAliArtists;
    function makeAliCollection(info) {
        var collectionOriginalId = info['listId'].toString();
        var collectionCoverUrl = url_1.getAbsolutUrl(info['collectLogo'], _staticResourceBasicUrl);
        var tags = (info['tags'] || []).map(function (tag) { return ({ name: tag }); });
        var songs = (info['songs'] || []).map(function (songInfo) { return makeAliSong(songInfo); });
        var duration = songs.length != 0 ? songs.map(function (s) { return s.duration; }).reduce(function (x, y) { return x + y; }) : null;
        var collection = {
            collectionId: _getCollectionId(collectionOriginalId),
            type: 'collection',
            origin: _origin,
            collectionOriginalId: collectionOriginalId,
            url: _getCollectionUrl(collectionOriginalId),
            collectionName: info['collectName'],
            collectionCoverUrl: collectionCoverUrl,
            userId: _getUserId((info['userId'] || '').toString()),
            userName: info['userName'],
            releaseDate: info['gmtCreate'],
            description: info['description'],
            tags: tags,
            duration: duration,
            songs: songs,
            songCount: info['songCount'],
            playCount: info['playCount'],
            likeCount: info['collects'],
        };
        return collection;
    }
    exports.makeAliCollection = makeAliCollection;
    function makeAliCollections(info) {
        var collections = (info || []).map(function (collectionInfo) { return makeAliCollection(collectionInfo); });
        return collections;
    }
    exports.makeAliCollections = makeAliCollections;
    function makeUserProfile(info) {
        var userOriginalId = info['userId'].toString();
        var userAvatarUrl = (info['avatar'] || '').split('@')[0];
        var user = {
            userId: _getUserId(userOriginalId),
            type: 'userProfile',
            origin: _origin,
            userOriginalId: userOriginalId,
            url: _getUserUrl(userOriginalId),
            userName: info['nickName'],
            userAvatarUrl: userAvatarUrl,
            followerCount: info['fans'] || null,
            followingCount: info['followers'] || null,
            listenCount: info['listens'] || null,
            description: info['description'],
        };
        return user;
    }
    exports.makeUserProfile = makeUserProfile;
    function makeUserProfiles(info) {
        return info.map(function (userProfile) { return makeUserProfile(userProfile); });
    }
    exports.makeUserProfiles = makeUserProfiles;
    function makeUserProfileMore(info) {
        var userFavoriteInfo = info['userFavoriteInfo'];
        var user = {
            userId: null,
            origin: _origin,
            type: 'userProfile',
            artistCount: userFavoriteInfo['artistCount'],
            albumCount: userFavoriteInfo['albumCount'],
            favoriteCollectionCount: info['userFavouriteCollectCount'],
        };
        return user;
    }
    exports.makeUserProfileMore = makeUserProfileMore;
    function makeAccount(info) {
        var userOriginalId = info['userId'].toString();
        var user = {
            userId: _getUserId(userOriginalId),
            type: 'userProfile',
            origin: _origin,
            userOriginalId: userOriginalId,
            userName: info['nickName'],
        };
        return {
            user: user,
            type: 'account',
            accessToken: info['accessToken'],
            refreshToken: info['refreshToken'],
        };
    }
    exports.makeAccount = makeAccount;
});
