define(["require", "exports", "chord/platform/utils/common/size", "chord/sound/common/origin"], function (require, exports, size_1, origin_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.makePodcasts = exports.makePodcast = exports.makeEpisodes = exports.makeEpisode = exports.makeAudio = void 0;
    var _origin = 'himalaya';
    var _getEpisodeId = origin_1.getEpisodeId.bind(null, _origin);
    var _getPodcastId = origin_1.getPodcastId.bind(null, _origin);
    var _getRadioId = origin_1.getRadioId.bind(null, _origin);
    var DOMAIN = 'https://www.himalaya.com';
    function makeEpisodeUrl(type, podcastName, podcastOriginalId, episodeName, episodeOriginalId) {
        var url = DOMAIN + '/' + type + '-podcasts/' + podcastName + '-' + podcastOriginalId + '/' + episodeName + '-' + episodeOriginalId;
        return url.toLowerCase().replace(/[- ]+/g, '-');
    }
    function makePodcastUrl(type, podcastName, podcastOriginalId) {
        var url = DOMAIN + '/' + type + '-podcasts/' + podcastName + '-' + podcastOriginalId;
        return url.toLowerCase().replace(/[- ]+/g, '-');
    }
    function makeAudio(url) {
        if (!url)
            return null;
        return {
            url: url,
            kbps: 64,
            format: 'mp3',
            size: 1,
        };
    }
    exports.makeAudio = makeAudio;
    function makeEpisode(info) {
        var episodeOriginalId = (info['trackId'] || info['id']).toString();
        var episodeName = info['title'] || info['trackName'];
        var podcastOriginalId = info['albumId'].toString();
        var podcastName = info['albumTitle'];
        var podcastCoverUrl = info['coverSmall'];
        var url = makeEpisodeUrl(info['secondCategory']['seoTitle'], podcastName, podcastOriginalId, episodeName, episodeOriginalId);
        var radioOriginalId = info['uid'].toString();
        var radioName = info['nickname'];
        var duration = info['duration'] * 1000;
        var audio = makeAudio(info['downloadUrl']);
        var audios = audio ? [audio] : [];
        var releaseDate = info['createdAt'];
        var episode = {
            episodeId: _getEpisodeId(episodeOriginalId),
            type: 'episode',
            origin: _origin,
            episodeOriginalId: episodeOriginalId,
            url: url,
            episodeName: episodeName,
            podcastId: _getPodcastId(podcastOriginalId),
            podcastOriginalId: podcastOriginalId,
            podcastName: podcastName,
            podcastCoverUrl: podcastCoverUrl,
            radioId: _getRadioId(radioOriginalId),
            radioOriginalId: radioOriginalId,
            radioName: radioName,
            description: info['intro'],
            // millisecond
            duration: duration,
            // millisecond
            releaseDate: releaseDate,
            playCountWeb: info['playtimes'],
            playCount: 0,
            likeCount: info['likes'],
            audios: audios,
            disable: !info['isFree'],
        };
        return episode;
    }
    exports.makeEpisode = makeEpisode;
    function makeEpisodes(info) {
        return (info || []).map(function (info) { return makeEpisode(info); });
    }
    exports.makeEpisodes = makeEpisodes;
    function makePodcast(info) {
        var podcastOriginalId = info['albumId'].toString();
        var podcastId = _getPodcastId(podcastOriginalId);
        var podcastName = info['title'];
        var podcastCoverUrl = info['coverOrigin'];
        var radioOriginalId = (info['anchorId'] || info['anchorUid'] || info['uid']).toString();
        var radioId = _getRadioId(radioOriginalId);
        var radioName = info['anchorName'] || info['anchorNickName'] || info['nickname'];
        var tags = [
            { name: info['firstCategory']['title'], id: info['firstCategory']['id'] },
            { name: info['secondCategory']['title'], id: info['secondCategory']['id'] },
        ];
        var url = makePodcastUrl(info['secondCategory']['title'], podcastName, podcastOriginalId);
        var podcast = {
            podcastId: podcastId,
            type: 'podcast',
            origin: _origin,
            podcastOriginalId: podcastOriginalId,
            url: url,
            podcastName: podcastName,
            podcastCoverUrl: podcastCoverUrl,
            radioId: radioId,
            radioName: radioName,
            description: info['intro'],
            tags: tags,
            releaseDate: info['createdAt'],
            episodes: [],
            episodeCount: info['tracks'],
            playCount: size_1.literal_to_number(info['playTimes']),
            likeCount: size_1.literal_to_number(info['subscribeCount']),
        };
        return podcast;
    }
    exports.makePodcast = makePodcast;
    function makePodcasts(info) {
        return (info || []).map(function (episodeInfo) { return makePodcast(episodeInfo); });
    }
    exports.makePodcasts = makePodcasts;
});
