define(["require", "exports", "chord/base/browser/proxy", "chord/unity/api/blocked", "chord/workbench/api/common/state/notification", "chord/workbench/parts/notification/action/notice", "chord/workbench/api/node/audio", "chord/music/common/origin", "chord/platform/log/common/log", "chord/platform/utils/common/paths", "chord/workbench/parts/player/browser/action/switchKbps", "chord/workbench/parts/player/browser/action/playList", "chord/workbench/api/utils/playItem", "chord/preference/configuration/app"], function (require, exports, proxy_1, blocked_1, notification_1, notice_1, audio_1, origin_1, log_1, paths_1, switchKbps_1, playList_1, playItem_1, app_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    var logger = new log_1.Logger(paths_1.filenameToNodeName(__filename));
    function switchLowerKbps() {
        var store = window.store;
        var state = store.getState();
        var index = state.player.index;
        var playItem = state.player.playList[index];
        var audio = playItem_1.selectAudio(playItem.audios);
        var preKbps = audio.kbps;
        logger.info('switch lower kbps:', playItem);
        if (switchKbps_1.switchKbps(-1)) {
            notice_1.noticeBlockedKbps(playItem, preKbps);
            playList_1.handlePlay(index).then(function (act) { return store.dispatch(act); });
        }
        else {
            notice_1.notice(notification_1.NOTICES.NO_AUDIO, playItem);
            // play next playItem
            playList_1.handlePlay(index + 1).then(function (act) { return store.dispatch(act); });
        }
    }
    /**
     * When radio url's domain is not from China, we try to use a proxy to request
     */
    function handleBlockedByISP() {
        var proxy = app_1.appConfiguration.getConfig().proxy;
        if (!proxy)
            return false;
        var store = window.store;
        var state = store.getState();
        var index = state.player.index;
        var playItem = state.player.playList[index];
        var audio = playItem_1.selectAudio(playItem.audios);
        if (!audio)
            return false;
        if (!blocked_1.hasChinaDomain(audio.url)) {
            proxy_1.setBrowserGlobalProxy(proxy);
            playList_1.handlePlay(index).then(function (act) { return store.dispatch(act); });
            return true;
        }
        return false;
    }
    function handleUrlExpire() {
        var store = window.store;
        var state = store.getState();
        var index = state.player.index;
        var playItem = state.player.playList[index];
        // Kuwo url has an expire time
        if (playItem.origin == origin_1.ORIGIN.kuwo) {
            playItem.audios.map(function (a) { return a.url = null; });
        }
        else {
            return false;
        }
        var audio = playItem_1.selectAudio(playItem.audios);
        if (!audio)
            return false;
        playList_1.handlePlay(index).then(function (act) { return store.dispatch(act); });
        return true;
    }
    /**
     * Switch to lower kbps for audio loading error
     */
    function onLoadError(soundId, store, audioUrl, playItemId) {
        audio_1.CAudio.destroy();
        // if (handleBlockedByISP()) return;
        if (handleUrlExpire()) {
            return;
        }
        switchLowerKbps();
    }
    // CAudio.registerOnPlay('player.onPlay', onPlay);
    audio_1.CAudio.registerOnLoadError('player.onLoadError', onLoadError);
});
