var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
define(["require", "exports", "react", "chord/workbench/parts/common/component/buttons", "chord/platform/utils/common/keys", "chord/workbench/api/node/audio", "chord/music/core/api", "chord/css!../media/lyric"], function (require, exports, React, buttons_1, keys_1, audio_1, api_1) {
    'use strict';
    Object.defineProperty(exports, "__esModule", { value: true });
    function getTimePoint() {
        return audio_1.CAudio.playing() ? audio_1.CAudio.seek() * 1000 : 0;
    }
    function NoLyricView() {
        return (React.createElement("div", { className: 'no-lyric-content' },
            React.createElement("div", { className: 'no-lyric' }, "NO LYRIC")));
    }
    var LyricView = /** @class */ (function (_super) {
        __extends(LyricView, _super);
        function LyricView(props) {
            var _this = _super.call(this, props) || this;
            _this.state = {
                lyric: null,
                currentIndex: 0,
                translation: false,
            };
            _this.init = true;
            _this.step = _this.step.bind(_this);
            _this.scrollToView = _this.scrollToView.bind(_this);
            _this.toggleTranslation = _this.toggleTranslation.bind(_this);
            _this.onplay = _this.onplay.bind(_this);
            _this.onpause = _this.onpause.bind(_this);
            _this.onend = _this.onend.bind(_this);
            _this.onseek = _this.onseek.bind(_this);
            _this.onloaderror = _this.onloaderror.bind(_this);
            _this.getLyric = _this.getLyric.bind(_this);
            // After mounting, let CAudio handle play & pause actions
            audio_1.CAudio.registerOnPlay('lyric-onplay', _this.onplay);
            audio_1.CAudio.registerOnPause('lyric-onpause', _this.onpause);
            audio_1.CAudio.registerOnEnd('lyric-onend', _this.onend);
            audio_1.CAudio.registerOnSeek('lyric-onseek', _this.onseek);
            audio_1.CAudio.registerOnLoadError('lyric-onloaderror', _this.onloaderror);
            return _this;
        }
        LyricView.prototype.componentDidMount = function () {
            // After getting lyric, an action is needed to call the step function,
            // so, let the param to be `true`
            this.getLyric(true);
        };
        LyricView.prototype.componentWillUnmount = function () {
            this.onpause();
        };
        /**
         * When playing song is changed, its lyric needs to fetch
         */
        LyricView.prototype.componentDidUpdate = function (preProps) {
            if (!this.props.song)
                return;
            if (this.props.song.songId != preProps.song.songId) {
                this.getLyric();
            }
        };
        /**
         * When the audio starts to playing or the LyricView is mounted, do the action
         */
        LyricView.prototype.onplay = function (nextFire) {
            if (nextFire === void 0) { nextFire = null; }
            if (!this.ppoint) {
                this.ppoint = setTimeout(this.step, nextFire);
            }
        };
        /**
         * When the audio starts to playing or the LyricView is mounted, do the action
         */
        LyricView.prototype.onpause = function () {
            if (this.ppoint) {
                clearTimeout(this.ppoint);
                this.ppoint = null;
            }
        };
        LyricView.prototype.onend = function () {
            this.onpause();
        };
        LyricView.prototype.onseek = function () {
            this.onpause();
            this.onplay();
        };
        LyricView.prototype.onloaderror = function () {
            this.onpause();
        };
        LyricView.prototype.toggleTranslation = function () {
            this.setState(function (preState) { return ({ translation: !preState.translation }); });
        };
        LyricView.prototype.scrollToView = function () {
            var elem = document.getElementById('lyric-this');
            if (elem)
                elem.scrollIntoView({ behavior: 'smooth', block: 'center', inline: 'nearest' });
        };
        LyricView.prototype.step = function () {
            var _this = this;
            if (!audio_1.CAudio.playing) {
                this.onpause();
                return;
            }
            var chunks = this.state.lyric.chunks;
            var timePoint = getTimePoint();
            var nextIndex = chunks.findIndex(function (chunk) { return timePoint <= chunk.point; });
            // No lyric text find
            if (nextIndex == -1) {
                this.onpause();
                return;
            }
            var currentIndex = nextIndex - 1;
            if (currentIndex < chunks.length - 1) {
                var nextFire = chunks[currentIndex + 1].point - timePoint;
                this.setState({ currentIndex: currentIndex });
                process.nextTick(function () { return _this.scrollToView(); });
                this.onpause();
                this.onplay(nextFire);
            }
        };
        LyricView.prototype.getLyric = function (init) {
            var _this = this;
            var song = this.props.song;
            if (!song) {
                // no lyric
                LyricView.thisLyric = null;
                this.setState({ lyric: null });
                return;
            }
            var thisLyric = LyricView.thisLyric;
            if (thisLyric && thisLyric.songId == song.songId) {
                this.setState({ lyric: thisLyric });
                if (init)
                    this.onplay();
                return;
            }
            api_1.musicApi.lyric(song.songId, song).then(function (lyric) {
                if (!lyric) {
                    LyricView.thisLyric = null;
                    _this.setState({ lyric: null, currentIndex: 0 });
                    return;
                }
                LyricView.thisLyric = lyric;
                _this.setState({ lyric: lyric, currentIndex: 0 });
            }).then(function () {
                if (init)
                    _this.onplay();
            });
        };
        LyricView.prototype.render = function () {
            var _this = this;
            if (this.init) {
                this.init = false;
                return React.createElement("br", null);
            }
            if (!this.state.lyric) {
                return (React.createElement(NoLyricView, null));
            }
            var currentIndex = this.state.currentIndex;
            var chunks = this.state.lyric.chunks;
            var chunksView = [];
            chunks.forEach(function (chunk, index) {
                if (chunk.type == 'breakline') {
                    chunksView.push(React.createElement("br", { key: keys_1.makeListKey(index, 'lyric', 'breakline') }));
                    return;
                }
                var active = index == currentIndex;
                chunksView.push(React.createElement("p", { key: keys_1.makeListKey(index, 'lyric', 'chunk'), id: active ? 'lyric-this' : undefined, className: active ? 'lyric-active' : 'lyric' }, chunk.text ? chunk.text : '\u00A0'));
                if (chunk.translation && _this.state.translation) {
                    chunksView.push(React.createElement("p", { key: keys_1.makeListKey(index, 'lyric', 'chunk', 'trans'), className: active ? 'lyric-active' : 'lyric' }, chunk.translation));
                }
            });
            var hasTranslation = chunks.some(function (chunk) { return !!chunk.translation; });
            var translationButton = hasTranslation ? (React.createElement("div", { style: { display: 'flex', justifyContent: 'center', margin: '3px 0px' } },
                React.createElement(buttons_1.SmallButton, { title: this.state.translation ? 'translated' : 'translate', click: this.toggleTranslation }))) : null;
            return (React.createElement("div", { className: 'lyric-content' },
                React.createElement("div", { className: 'lyric-content-container' }, chunksView),
                translationButton));
        };
        return LyricView;
    }(React.Component));
    exports.default = LyricView;
});
